/*************************************************************************
Title:    AVR, ATMEGA8 @ x MHz
Author:   Levas Vabolis
File:     main.c
Software: AVR-GCC x.x
Hardware: universal PCB mega8
Fuses:	
Description: primitive high voltage AVR programmer. For pile of ATMEGA162s
Web: http://www.vabolis.lt
**************************************************************************/

#define F_CPU   11000000L //not sure

#include <avr/io.h>
#include <util/delay.h>

// not sure about default fuses
#define FUSE_HI 0b10011001 //0x99
//#define FUSE_HI 0x00
#define FUSE_LO 0b01100010 //0x62
#define FUSE_EXT 0xff

/* fuses
HIGH
7 OCDEN 1 <--off
6 JTAGEN 0 <--on
5 SPIEN 0
4 WDTON 1
3 EESAVE 1
2 BOOTSZ1 0
1 BOOTSZ0 0
0 BOOTRST 1

LOW
7 CKDIV8 0
6 CKOOUT 1
5 SUT1 1
4 SUT0 0
3 CKSEL3 0
2 CKSEL2 0
1 CKSEL1 1
0 CKSEL0 0

EXT
7 na 1
6 na 1
5 na 1
4 M161C 1
3 BODLEVEL2 1
2 BODLEVEL1 1
1 BODLEVEL0 1
0 na 1

*/ 

void inithardware(void)
{
// *** DATA PORTC, pin to pin from master to slave. PC<-->PB, execp two pins, B6 and B7
DDRC=0xFF; // DATA OUT, to MEGA162 PB
PORTC=0;
#define PDATA PORTC


// *** CONTROL PORT PORTD. Almoust all pins are pin2pin. PD1-PD7 <--> PD1-PD7
// WARNING. PORTD PIN0 is RESET mosfet control. When high, mosfet is open, and SLAVE RESET is 0V.
// When pin is low, mosfet is OFF and slave RESET pint is +12V.

DDRD=0b11111101; // control out. RDY for input. RDY/BUSY not used in this software, no need to connect.

#define RESET _BV(0)
#define RESET_LO PORTD&=~RESET
#define RESET_HI PORTD|=RESET
#define RDY_BUSY _BV(1)
#define OE _BV(2)
#define OE_LO PORTD&=~OE
#define OE_HI PORTD|=OE
#define WR _BV(3)
#define WR_LO PORTD&=~WR
#define WR_HI PORTD|=WR
#define BS1 _BV(4)
#define BS1_LO PORTD&=~BS1
#define BS1_HI PORTD|=BS1
#define XA0 _BV(5)
#define XA0_LO PORTD&=~XA0
#define XA0_HI PORTD|=XA0
#define XA1 _BV(6)
#define XA1_LO PORTD&=~XA1
#define XA1_HI PORTD|=XA1
#define PAGEL _BV(7)
#define PAGEL_LO PORTD&=~PAGEL
#define PAGEL_HI PORTD|=PAGEL
PORTD=RESET+OE+WR; //reset high (slave reset 0), /OE-high, /WR-high

// *** PORTB, other connections and START button (PIN B0)
// PINB1->BS2--> slave PA0
// PINB2->XTAL1-->slave XTAL1 quartz oscilatori input pin.
// missing data pins from PORTC are here PPB6 and PPB7

#define BS2 _BV(1)
#define BS2_LO PORTB&=~BS2
#define BS2_HI PORTB|=BS2

#define XTAL1 _BV(2)
#define XTAL1_LO PORTB&=~XTAL1
#define XTAL1_HI PORTB|=XTAL1
#define XTAL1_TOGLE PORTB^=XTAL1


#define PPB6 _BV(3)
#define PPB6_LO PORTB&=~PPB6
#define PPB6_HI PORTB|=PPB6
#define PPB7 _BV(4)
#define PPB7_LO PORTB&=~PPB7
#define PPB7_HI PORTB|=PPB7

#define DBG _BV(5)
#define DBG_LO PORTB&=~DBG
#define DBG_HI PORTB|=DBG

DDRB=BS2+XTAL1+PPB6+PPB7+DBG;
#define BUTTON _BV(0)
PORTB=BUTTON; //pull-up
}

void pulse_XTAL1(void)
{ XTAL1_LO; XTAL1_HI; _delay_us(1); XTAL1_LO; }

void pulse_WR(void)
{ WR_HI; WR_LO; _delay_us(1); WR_HI; }

void setdata(unsigned char n)
{
PDATA=n & 0b00111111;
if(n & _BV(7)) {PPB7_HI;} else {PPB7_LO;}
if(n & _BV(6)) {PPB6_HI;} else {PPB6_LO;}
}

void enter_mode(void)
{
unsigned char i;

//initial default settings
//entering programming mode
setdata(0);
WR_HI; 		// set write to default high
OE_HI; 		//no output, write only
PAGEL_LO;
XA1_LO;
XA0_LO;
BS1_LO;
BS2_LO;
XTAL1_LO;

RESET_HI;   	// slave reset pin low

for(i=0;i<20;i++) { XTAL1_TOGLE; _delay_us(1);} // toggle XTAL1 at least 6 times
XTAL1_LO;
_delay_us(5);


	// set PAGEL, XA1, XA0, BS1 to 0000, set in the beginning.
PAGEL_LO;
XA1_LO;
XA0_LO;
BS1_LO;
RESET_LO;	// slave reset pin +12V
_delay_us(1);
}


void chip_erase(void)
{


//chip erase sequence
XA1_HI;
XA0_LO; // enable command

BS1_LO;
setdata(0b10000000); 	//command chip erase

pulse_XTAL1(); 		//xtal1 positive pulse
pulse_WR(); 		//execute command. Now it is erasing chip. RDY/BSY must go low.
//wait for RDY/BSY goes high...
_delay_ms(20); 		// max 10ms from datasheet
XA1_LO;
XA0_LO;

_delay_ms(1); 
}

void chip_fuse_LO(void)
{

//enter_mode();

//WRITE FUSE LOW
XA0_LO; BS1_LO;
setdata(0x40);
XA1_HI;
pulse_XTAL1();
XA1_LO; XA0_HI;
setdata(FUSE_LO);
pulse_XTAL1();
pulse_WR();
_delay_ms(6);
}
void chip_fuse_HI(void)
{

//enter_mode();
//WRITE FUSE HI
XA0_LO; BS1_LO;
setdata(0x40);
XA1_HI;
pulse_XTAL1();
XA1_LO;  XA0_HI;
setdata(FUSE_HI);
pulse_XTAL1();
BS1_HI;
pulse_WR();
_delay_ms(6);
BS1_LO;
}
void chip_fuse_EXT(void)
{
//enter_mode();
//WRITE FUSE EXT
XA0_LO; BS1_LO;
setdata(0x40);
XA1_HI;
pulse_XTAL1();
XA1_LO;  XA0_HI;
setdata(FUSE_EXT);
pulse_XTAL1();
BS2_HI;
pulse_WR();
_delay_ms(6);
BS2_LO;
}


void chip_idle(void)
{
setdata(0);

// END FUSE

_delay_ms(1);

RESET_HI;   // slave reset pin low
_delay_ms(500);
_delay_ms(500);
_delay_ms(500);
}


int main(void)
{

inithardware();
for(;;)
	{
	if(~PINB & BUTTON){
	
	_delay_us(100);
DBG_HI;
	enter_mode();
	chip_erase();
	chip_fuse_LO();
	chip_fuse_HI();
	chip_fuse_EXT();
	chip_idle();
DBG_LO;
		}
	}
}
/* Programinis dugnas .....abyss   */
